package cz.muni.makub.model;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.RowMapper;
import org.springframework.jdbc.core.simple.ParameterizedSingleColumnRowMapper;
import org.springframework.stereotype.Repository;

import javax.sql.DataSource;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.List;

/**
 * Implementace manažera knih. Anotace @Repository oznamuje Springu, že má vytvořit instanci této třídy
 * a zapamatovat si ji jako Spring-bean pod jménem "bookManager". Anotace @Autowired říká Springu,
 * že má metodu setDataSource() zavolat a předat jí Spring-bean jménem "dataSource".
 * Ten je definován v konfiguračním souboru Springu tj. ve WEB-INF/spring-context.xml.
 * <p/>
 * Implementace této třídy využívá knihovnu Spring JDBC. Třída ParameterizedBeanPropertyRowMapper
 * mapuje sloupce z ResultSet na stejnojmené properties JavaBenu, je tedy důležité, aby se jména
 * shodovala. Jinak by bylo nutné vytvořit implementaci ParameterizedRowMapper mapující
 * sloupce ručně.
 *
 * @author Martin Kuba makub@ics.muni.cz
 */
@Repository("bookManager")
public class BookManagerImpl implements BookManager {

    final static Logger log = LoggerFactory.getLogger(BookManagerImpl.class);

    public BookManagerImpl() {
        log.debug("vytvořen");
    }

    private JdbcTemplate jdbc;

    @Autowired
    public void setDataSource(DataSource dataSource) {
        log.debug("dataSource set to {}", dataSource);
        jdbc = new JdbcTemplate(dataSource);
    }

    //mapuje data z ResultSet na třídu Book
    private static final RowMapper<Book> BOOK_MAPPER = new RowMapper<Book>() {
        @Override
        public Book mapRow(ResultSet rs, int i) throws SQLException {
            Book book = new Book();
            book.setId(rs.getInt("id"));
            book.setName(rs.getString("name"));
            book.setAuthor(rs.getString("author"));
            book.setPrice(rs.getInt("price"));
            book.setPaperback(rs.getBoolean("paperback"));
            book.setColor(Book.Color.valueOf(rs.getString("color")));
            return book;
        }
    };

    //mapuje jediný sloupec z ResultSet na String
    private static final ParameterizedSingleColumnRowMapper<String> STRING_MAPPER = ParameterizedSingleColumnRowMapper.newInstance(String.class);


    public List<Book> getAllBooks() {
        return jdbc.query("select * from books order by id", BOOK_MAPPER);
    }

    public Book getBookById(int id) {
        return jdbc.queryForObject("select * from books where id = ?", BOOK_MAPPER, id);
    }

    public void createBook(Book book) {
        jdbc.update("insert into books (name,author,paperback,color,price) values (?,?,?,?,?)",
                book.getName(), book.getAuthor(), book.isPaperback(), book.getColor().name(), book.getPrice());
        
    }

    public void updateBook(Book book) {
        jdbc.update("update books set name=?,author=?,paperback=?,color=?,price=? where id=?",
                book.getName(), book.getAuthor(), book.isPaperback(), book.getColor().name(), book.getPrice(), book.getId());
    }

    public void deleteBook(Book book) {
        jdbc.update("delete from books where id = ?", book.getId());
    }

    public List<String> findBookNamesBySubstring(String substr) {
        return jdbc.query("select name from books where name like ?", STRING_MAPPER, "%" + substr + "%");
    }

    public List<Book> findBooksByName(String name) {
        return jdbc.query("select * from books where name like ?", BOOK_MAPPER, "%" + name + "%");
    }


}
